##
## Voices for NVDA
## Copyright (c) 2015 by Code Factory, S.L. All rights reserved.
## Eloquence and Vocalizer Expressive are registered trademarks
## of Nuance Communications, Inc.
##


import os
from collections import OrderedDict
import Queue
import threading
import languageHandler
from synthDriverHandler import SynthDriver,VoiceInfo,BooleanSynthSetting
import speech
from logHandler import log
import _cfnvdainterface as cfni



minRate = -10
maxRate = 10
minPitch = -10
maxPitch = 10
minVolume = 0
maxVolume = 100

def  in_range(val, min, max ):
    if val < min:
        return min
    if val > max:
        return max
    return val

class genericSynthDriver(SynthDriver):

    @classmethod
    def check(cls):
        return True

    def __init__(self):
        cfni.initialize(self.name, self.dllname_,self.exename_)
        self._language=languageHandler.getLanguage()
        self._rate=0
        self._pitch=0
        self._volume=100
        self._inflection=75

        try:
            self._set_voice(self._getAvailableVoices().keys()[0])
        except:
            pass

    def _processText(self, text):
        text = unicode(text)
        # We need to make several replacements.
        return text.translate({
            0x1: None, # used for embedded commands
            0x3C: u"&lt;", # <: because of XML
            0x3E: u"&gt;", # >: because of XML
        })
    
    def speak(self,speechSequence):
	textList=[]
        closeTags = []
        ## Pitch control
        
	for item in speechSequence:
	    if isinstance(item,basestring):
		textList.append(self._processText(item))
	    elif isinstance(item,speech.IndexCommand):
		textList.append("<BOOKMARK mark='%d'/>"%item.index)
	    elif isinstance(item,speech.CharacterModeCommand):
                textList.append("<spell>")
                closeTags.append("</spell>")
                pass
	    elif isinstance(item,speech.LangChangeCommand):
                if item.lang != self._language:
                    textList.append("<lang langid='%s'>" % self._LangNVDAToCF(item.lang))
                    closeTags.append("</lang>")
                pass
	    elif isinstance(item,speech.BreakCommand):
		textList.append("<silence msecs='%d'/>" % item.time)
	    elif isinstance(item,speech.PitchCommand):
		textList.append("<pitch middle='%d'/>" % self._PitchNVDAToCF(item.time))
                closeTags.Append("</pitch>")
	    elif isinstance(item,speech.RateCommand):
		textList.append("<rate abspeed='%d'/>" % self._RateNVDAToCF(item.time))
                closeTags.append("</rate>")
	    elif isinstance(item,speech.VolumeCommand):                
		textList.append("<volume level='%d'>" % self._VolumeNVDAToCF(item.time))
                closeTags.append("</volume>")
	    elif isinstance(item,speech.SpeechCommand):
		log.debugWarning("Unsupported speech command: %s"%item)
	    else:
		log.error("Unknown speech: %s"%item)
        ##Now just play out the composed string.
        closeTags.reverse()
	text=u"".join(textList + closeTags)
	cfni.speak(text)

    def _LangNVDAToCF( self, lang):
        return lang.lower()
    
    def _PitchNVDAToCF( self, mult):
        return in_range(mult * self._pitch,minPitch,maxPitch)
    
    def _RateNVDAToCF( self, mult):
        return in_range(mult * self._rate, minRate, maxRate)
    
    def _VolumeNVDAToCF( self, mult):
        return in_range(mult * self._volume, minVolume, maxVolume)
    
    def terminate(self):
        cfni.terminate()

    def cancel(self):
        cfni.stop()

    def pause(self,switch):
        cfni.pause(switch)


    def _getAvailableVoices(self):
	voices=OrderedDict()
	for v in cfni.getVoiceList():
	    voices[v[0]]=VoiceInfo(v[0],v[1],v[2])
	return voices

    def _get_voice(self):
        vc = getattr(self,'_voice',None)
        if vc : return vc
        try:
            vc = self._getAvailableVoices().keys()[0]
            return vc
        except:
            return ''


    def _set_voice(self, identifier):
	if not identifier:
	    return
        cfni.setVoice(identifier)
	self._voice=identifier
	#self._language=super(SynthDriver,self).language

    ## If implementing VariantSetting ( Eloq. only )
    def _getAvailableVariants(self):
	variants=OrderedDict()
	for v in cfni.getVariantList():
	    variants[v[0]]=VoiceInfo(v[0],v[1])
	return variants

    def _get_variant(self):
        return  getattr(self, '_variant', "")

    def _set_variant(self,variant):
        if not variant:
            return
        self._variant = variant
        cfni.setVariant(variant)

    def _get_rate(self):
        realrate =  getattr(self,'_rate',0)
        return self._paramToPercent(realrate, minRate, maxRate)

    def _set_rate(self,rate):
        r = self._percentToParam(rate,minRate,maxRate)
        self._rate = in_range(r, minRate, maxRate)
        cfni.setRate(self._rate)

    def _get_pitch(self):
        realpitch =  getattr(self,'_pitch',0)
        return self._paramToPercent(realpitch, minPitch, maxPitch)

    def _set_pitch(self,pitch):
        p = self._percentToParam(pitch,minPitch,maxPitch)
        self._pitch = in_range(p, minPitch, maxPitch)
        cfni.setPitch(self._pitch)
        
    def _get_volume(self):
        realvolume =  getattr(self,'_volume',100)
        return self._paramToPercent(realvolume, minVolume, maxVolume)

    def _set_volume(self,volume):
        self._volume = self._percentToParam(volume,minVolume,maxVolume)
        cfni.setVolume(self._volume)

    def _get_language(self):
        return self._language

    def _get_lastIndex(self):
        return cfni.lastIndex

        
